/*******************************************************************************
 * Copyright (c) 2014, 2017 IBM Corp.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * and Eclipse Distribution License v1.0 which accompany this distribution.
 *
 * The Eclipse Public License is available at
 *    http://www.eclipse.org/legal/epl-v10.html
 * and the Eclipse Distribution License is available at
 *   http://www.eclipse.org/org/documents/edl-v10.php.
 *
 * Contributors:
 *    Allan Stockdill-Mander - initial API and implementation and/or initial documentation
 *    Ian Craggs - return codes from linux_read
 *******************************************************************************/

#include "MQTTLinux.h"
#include "../../../OneNETMqttClient/OneNETMqttClient.h"
#include "mbedtls/net_sockets.h"
#include "mbedtls/debug.h"
#include "mbedtls/ssl.h"
#include "mbedtls/entropy.h"
#include "mbedtls/ctr_drbg.h"
#include "mbedtls/error.h"
#include "mbedtls/certs.h"
#include "mbedtls/sha1.h"
#include "mbedtls/sha256.h"




#define DBG printf
typedef struct {
    mbedtls_ssl_context ssl_ctx;        /* mbedtls ssl context */
    mbedtls_net_context net_ctx;        /* Fill in socket id */
    mbedtls_ssl_config ssl_conf;        /* SSL configuration */
    mbedtls_entropy_context entropy;
    mbedtls_ctr_drbg_context ctr_drbg;
    mbedtls_x509_crt_profile profile;
    mbedtls_x509_crt cacert;
    mbedtls_x509_crt clicert;
    mbedtls_pk_context pkey;
} tls_t;


const char client_pk[] =
{
	"-----BEGIN PRIVATE KEY-----\r\n"
	"MIIEvAIBADANBgkqhkiG9w0BAQEFAASCBKYwggSiAgEAAoIBAQC/X1oERnriKcx8\r\n"
	"eU6pQQmysOhSNFE3VKEUeRAn8Qjdd2iuJcBnHlze98DOp6jlu5RnvMy9jbqQ9i9/\r\n"
	"d7WwmUZLsMuKWAJLyHxdyXNovp2peuMJlMd+Q56BioS6jU2WuO31DNwLwXNoAKYe\r\n"
	"4RRhIeDZ3u/kN3umZ1JXukeVXc17wXkCHUeQd15saDE9B+RQtP886Zssjz/JVegj\r\n"
	"s1cbkh1mG56XC5OSaYTeMgJef72DXwwP0o+W9wkQsLsMHcX30BzyOLgKyyFo8L4o\r\n"
	"EhZWZ4ghelatygAK9v1almT4S6CPzHaa8ga90QF075n3OBYydZZlYcgnSM1fvRs/\r\n"
	"Mb4TW71HAgMBAAECggEAcGD5Bmuw+6CaETpXI/MqU8GpG6TXUIuk79XL0/iqQQs+\r\n"
	"8I0AuE1PMiwBEBnKh+WoV2a6QFG7DidTayIMEsFfU1FlpT7bUk9QIO8Mk6TkBsNA\r\n"
	"Eb9Jiu8AZIfIO5iAyJKeQfmkOk8GizteUeHQyt1PIgHVk4m1Vdl0tEmSgbcM1K+a\r\n"
	"5bjSTs5NEkpehsTP7hWLJO2ALd+rFuveLsLWYr00oblZeIfFOIxlU5w24L6vX/h7\r\n"
	"uF7hvz4AX6p66b7jXB2MxdHwwFDsSaOWAXVIAMB0rHC8QTGVats5FkN+JwWTLo5n\r\n"
	"UtO+payxR4s+ZNgdkJoc/Yn9x4nNY1UgU3l+4KyeSQKBgQDOx30iInC/0M/upuIK\r\n"
	"ivbKd2ZkUwcaDwO+o5NmbkOqEPlWTzMhjpbfNFptvASY61ukrGmcWBU2QsSPzvA3\r\n"
	"Fd2DGEmIIiyeMyBNFH01SsPdZp1RgHjYggbaMQxRL3ePehUoWHoeatSpohMJDfMH\r\n"
	"sfloyMCPcAm3mVq+oSISBtLtJQKBgQDs7QTsRaoa7AilWNP3ntG/fqG+90fDcraL\r\n"
	"vtiGH1EaKDamOF1kium8oe+eEvYtLVZIT7kNVW8Sx0BpcknUBCcNYu0kJXXD78AK\r\n"
	"VkI7dxCtFd2pLKqLuu4FvMvUNxDxp02e1UUHwin9LsgDEIoYLPqN8pRjSOMHgXCo\r\n"
	"uZaQgIwy+wKBgECN7vb1IZYArqmm1/LKp3lWBVVUxmbQQS8HCkF0MxOo8O6NEz1q\r\n"
	"spdEVOblR9olwBqM6PuyVAz4kz0CMeRAo0MGFg61/I/3DiKCPwSgLr20B3EmVpPh\r\n"
	"JBqCLYI6+2yDhI9v79AHMhUzUBYHkNPaMC8SXWtqYt46tiXFUEmob7kxAoGAJ6eQ\r\n"
	"EF+klfb1h38zkxlg79UpC0ZmG39isEcUBHbPYzph0zZR41cgwfK0oVd5Hpyc+WNm\r\n"
	"jKXE/nadAEkhtDuxsZriKA+hA1doZBKF6WAHSYEEZaFVNifmWM19rBo/WsLuyevW\r\n"
	"LvM/heHDlke5ZUx57zIvxeJYJVRz37mYSr1JOesCgYA1Ieqhqklrwr3/uEcLeSTY\r\n"
	"MVipMMaxSETBog7QHg8sreauO+A7MltIWaDz+2ca/RM78KRV/OAQfIGAqHWFdiA3\r\n"
	"4rBgTebuxz3csFP/rsKS3g2+PRZEQfg917OkKjFWrnuwgvD1L0GbRldquLY1qf3s\r\n"
	"2/GPtr0qGycQO7Kmgz2diw==\r\n"
	"-----END PRIVATE KEY-----\r\n"
};
const char client_cert[] =
{
	"-----BEGIN CERTIFICATE-----\r\n"
	"MIIEdzCCAl+gAwIBAgIQFEYsglSJOh/AF7uvRz14RjANBgkqhkiG9w0BAQsFADA/\r\n"
	"MQswCQYDVQQGEwJDTjESMBAGA1UECAwJQ2hvbmdRaW5nMQ0wCwYDVQQKDARlZGdl\r\n"
	"MQ0wCwYDVQQDDARlZGdlMB4XDTIwMDYyMzA3MDQwOFoXDTIxMDYyMzA3MDQwOFow\r\n"
	"TjEYMBYGA1UEChMPY2hpbmFtb2JpbGUuY29tMRgwFgYDVQQLEw9jaGluYW1vYmls\r\n"
	"ZS5jb20xGDAWBgNVBAMTD2NoaW5hbW9iaWxlLmNvbTCCASIwDQYJKoZIhvcNAQEB\r\n"
	"BQADggEPADCCAQoCggEBAL9fWgRGeuIpzHx5TqlBCbKw6FI0UTdUoRR5ECfxCN13\r\n"
	"aK4lwGceXN73wM6nqOW7lGe8zL2NupD2L393tbCZRkuwy4pYAkvIfF3Jc2i+nal6\r\n"
	"4wmUx35DnoGKhLqNTZa47fUM3AvBc2gAph7hFGEh4Nne7+Q3e6ZnUle6R5VdzXvB\r\n"
	"eQIdR5B3XmxoMT0H5FC0/zzpmyyPP8lV6COzVxuSHWYbnpcLk5JphN4yAl5/vYNf\r\n"
	"DA/Sj5b3CRCwuwwdxffQHPI4uArLIWjwvigSFlZniCF6Vq3KAAr2/VqWZPhLoI/M\r\n"
	"dpryBr3RAXTvmfc4FjJ1lmVhyCdIzV+9Gz8xvhNbvUcCAwEAAaNgMF4wDgYDVR0P\r\n"
	"AQH/BAQDAgWgMB0GA1UdJQQWMBQGCCsGAQUFBwMCBggrBgEFBQcDATAMBgNVHRMB\r\n"
	"Af8EAjAAMB8GA1UdIwQYMBaAFPq/GiiUkm9SQ37P8ZTsP055+IrbMA0GCSqGSIb3\r\n"
	"DQEBCwUAA4ICAQC73DVjUy/F//tUSNxCOleqddKGYWQgOrwgHuiZSd6Nf0Wep/s5\r\n"
	"c23X1K8bz+O/8im6gXm2xpBMhW0qEZg2KNgcaOnJKlVWEQ179HUpnCIxhxxoaqYf\r\n"
	"OvPuG2ca4KGAGu8LXsom0jmGhj5ikGF3JHVVVDSe37ERIDpuBLBLSgKVFYkhaft/\r\n"
	"28CaV0PFA5Mfhd1YEuBAZclcT4+isOchzzUFHCRJBxoKjqyR0Qg7YTOYlhe9CLR4\r\n"
	"dcTa/g3sFbcb7EMOQmxpliU06q2oGyXq+I36dVKpWpSVMB1J8kJ9E54dCipVuJJv\r\n"
	"eWDSd7yeX4qNp56EFDMWMzaTjwhVgtzikhKCcuub97KmpbzTGcHZtBdWOoKTvKzw\r\n"
	"vvK03HTFbMkZC/CmY8emBbUTx4Ofg7Mnw8rqYuWhrZEfMkk7Kx4C2dhaR4E4Lheb\r\n"
	"K8pUWPHDtpgZeThbf4WDEWboAiV9ei0htIDv1Yv00Y+iAjhAiK6/nV47BoBwUjOJ\r\n"
	"hRLObxx1J5+qsDLGR4viZ/e9P9x4Il7kMjX1AZmnvb2DoHvpgqCuDXWTqXsUojzD\r\n"
	"+6MRbrIrQSk3S1BPUZgcNAPrEieSVpkT7cZABUOYcCQ+ZfCb4C4/zCHHHr8zDog3\r\n"
	"XGFoT4+ZwfKpj7YhyDUBak0BqHPD3wDoQsmALh5Ub4KNw/oW3CLpGy6ruQ==\r\n"
	"-----END CERTIFICATE-----\r\n"
};




const char client_pk1[] =
{
"-----BEGIN PRIVATE KEY-----\r\n"
"MIIEvQIBADANBgkqhkiG9w0BAQEFAASCBKcwggSjAgEAAoIBAQDMsO9VMB29gq9h\r\n"
"UjCip1Bq14IKqCK9ow6/Y4vk0eGS6EENXBt5J+imrPmRy4tGFJFr6abXDANgK05V\r\n"
"hNjXYxbyXJtAhzAOn9Z3FApbpdUA5nTVi/uWwgbHgt2a2ejIwASeBy4dfA4bSiqy\r\n"
"Ot3BmavIAty9YDetwqNiJkSXYaJ8MZtnEoyfm7f6OptSR5ODP+bgP3OvrwMfUmHD\r\n"
"sWrMq1PDTNsEXtycrsXAp22ryAjkdUlw4yPOc3ckmDtse2enGQOqurzN4Z8N0jTj\r\n"
"0WQ5UVe92K5p+s39m5F2nBjmjn8RI5VeevvxXUq31V4p99fAZXJUwrHvKnRgmiWY\r\n"
"09UtHBifAgMBAAECggEABnsXqM/UMTrhCKjMh2navvdB08qNBwoRnnetZzpJ9RfL\r\n"
"b3vaFtTvpc9WO4qBR1/BVvRX5vknZjUYztZc8zzc7iGGfN2K5Pukk7KbvVkbzGJ/\r\n"
"kn9TIOvVwR1gnNWHbEvAGazKhf52h+CqR2hbuxR9fuCwUoKoYKUH/USO6Wga9+mf\r\n"
"rgbHwVOG296C/pT8eHDj6l19MYZTW0tfDy8OJEEqfC3XwQMn9luZyHFTS9zLI9v5\r\n"
"T+I/YyEx9XMvQg4lHfZSTnjWe7JB/3+d+zO1Bh1e6jIFktp0I5c5QkLcvKQG35x9\r\n"
"OHDjOWYtPBg0FxQGOipDhdErvPUQfl60XE84k4MJcQKBgQDmXlcbv2r1mUEY+KNB\r\n"
"LOssp1dMTa+RffjZYsLxUw3nxXAMu1IwWQW4jBXU9NRk3U+kwTLapmDaLCk4GPjG\r\n"
"vUSHn69A+jTH1Pl0hNqBYuNi3BeJmP19sp5peczdG8LsCN6KP1EwjID8sXoho6jK\r\n"
"rJmodQwFq7MKhMsdiAPKW6G0VwKBgQDjdzfUDgz1SIA/9rLfjBPJ+4pSMRPwQiOT\r\n"
"KsGmWEQhraEZ7mGOl1yK6ubBpsdZbgbHXy4X4kt6ORBMVW882CItO6LeVhzmzBQo\r\n"
"hT+mCVWGZInHEWMZBCC00fvlHzY6nj9EAWq68y8iMZs+llVOMfZEf0X244GZDp04\r\n"
"y+YET/vQ+QKBgFjY8cel396xKMY0vuIYLOuf7q9jFt+KMjvFN2JTof1NzJVceapj\r\n"
"6yzt+k2Tz4bHRDxKtw5VNO1wMsWL/AbKYG/eSn1JAqo/naYuxwc32HL8Z6eN9dZQ\r\n"
"2X/clgKA/oCCOzIekXWj+ut6l5v5vtPMFcc+vVBClF7uMFhHaQ7WoFspAoGBAJyT\r\n"
"/1k4htD9pFLjhgP+D801GuUzJW94h7aRcxYHonawZ7pK7trR9IEn68qX/Jfh9kad\r\n"
"jUQhoSU0TLTPBrCScwxCBf4YtvJkvueZudOYj0fgwzrGDDo3LjBqb4Qr9Q9qyVAN\r\n"
"zaN9LQ/Tp/LwVy5plAVYuxeVSivK9I0ZRm/ngwc5AoGAPFfPEp26iFtWLC4J+eXI\r\n"
"GiOOrjGlDQwj86YR6sYeTlVR0d54TcC6mNhZW9I9KVXPvB7+2QGYIyv/LCiSIcg6\r\n"
"qh9WbU6imiwEgp31t74G+LsXgo+UU1Sv5IsBC3N2hr32A8EB60QpB6lfS1mrJ62g\r\n"
"qfnZhqXTgVaOmkB68Qj7qt4=\r\n"
"-----END PRIVATE KEY-----\r\n"
};

const char client_cert1[] =
{
"-----BEGIN CERTIFICATE-----\r\n"
"MIIEeDCCAmCgAwIBAgIRAOfpCXqND1nuvv8jPgBmYnowDQYJKoZIhvcNAQELBQAw\r\n"
"PzELMAkGA1UEBhMCQ04xEjAQBgNVBAgMCUNob25nUWluZzENMAsGA1UECgwEZWRn\r\n"
"ZTENMAsGA1UEAwwEZWRnZTAeFw0yMDA0MDIwODAxMjJaFw0yMTA0MDIwODAxMjJa\r\n"
"ME4xGDAWBgNVBAoTD2NoaW5hbW9iaWxlLmNvbTEYMBYGA1UECxMPY2hpbmFtb2Jp\r\n"
"bGUuY29tMRgwFgYDVQQDEw9jaGluYW1vYmlsZS5jb20wggEiMA0GCSqGSIb3DQEB\r\n"
"AQUAA4IBDwAwggEKAoIBAQDMsO9VMB29gq9hUjCip1Bq14IKqCK9ow6/Y4vk0eGS\r\n"
"6EENXBt5J+imrPmRy4tGFJFr6abXDANgK05VhNjXYxbyXJtAhzAOn9Z3FApbpdUA\r\n"
"5nTVi/uWwgbHgt2a2ejIwASeBy4dfA4bSiqyOt3BmavIAty9YDetwqNiJkSXYaJ8\r\n"
"MZtnEoyfm7f6OptSR5ODP+bgP3OvrwMfUmHDsWrMq1PDTNsEXtycrsXAp22ryAjk\r\n"
"dUlw4yPOc3ckmDtse2enGQOqurzN4Z8N0jTj0WQ5UVe92K5p+s39m5F2nBjmjn8R\r\n"
"I5VeevvxXUq31V4p99fAZXJUwrHvKnRgmiWY09UtHBifAgMBAAGjYDBeMA4GA1Ud\r\n"
"DwEB/wQEAwIFoDAdBgNVHSUEFjAUBggrBgEFBQcDAgYIKwYBBQUHAwEwDAYDVR0T\r\n"
"AQH/BAIwADAfBgNVHSMEGDAWgBT6vxoolJJvUkN+z/GU7D9OefiK2zANBgkqhkiG\r\n"
"9w0BAQsFAAOCAgEAbaXJ8EmSxY8DqkGtr7iIY9VQkp/N3cD4vOvBTR9l+vNZtk6m\r\n"
"GSOVi02ClWuAkj7HQdAjip1XLAiINqO+ZPhogBGQceCZTL1ifDfhKIFxv1iOsZEU\r\n"
"toCHSDZQmoSf4mXih5vBf7M1nGHTozITj2oh9brwYmiVHJZDNb7fgneXUfOwQsKh\r\n"
"SXrtAiHmk8+8aFCACCcteMgU2TifzcbU9QaXLGiqjWzCZ3rOL7++ZWvYnmK+VkOG\r\n"
"mBslqKlNoq5aBay5+Rb447aF5S4ArwqInR7unke0gFWQp49wYpk6CLPTpxk3zI0S\r\n"
"hMOUlfdObK8ozmVAhlKCt/w+tP98ym/lgHlvs3nGovwu4rzZ3us1EbyqxznRlrsn\r\n"
"qXBU9an7pnxey90v9J8MT2fy5X1/FC9EtynDTKn2XpZyopm1GXgxjCjS/mh8yygA\r\n"
"p4WWi1RFAZ52YPf0fbG36Clie7y+1+rIRbQMs6wSVgnT8bqOWALrOnA+6bqbMLb4\r\n"
"D7sfTamAChZqzzoZYZRH/+mFZnblBEgdnrFy59GBvbSd8bdW2Y+mstI8AWDwpIII\r\n"
"nj2jKQ5LnGVCnuXgYlQ8VRbNSep6yRO/GCHTjacpnPBq+dxlkgnAmg9jo8vD2ie2\r\n"
"NHLTTreOIJmZuNtySCPO2H/zy8Ih+gVP8dYV+dkeXgbXlwWtTxZNPNQGr7w=\r\n"
"-----END CERTIFICATE-----\r\n"
};

const char ca_cert[] = 
{
"-----BEGIN CERTIFICATE-----\r\n"
"MIIFZDCCA0ygAwIBAgIJAIHCcQYLQY7sMA0GCSqGSIb3DQEBCwUAMD8xCzAJBgNV\r\n"
"BAYTAkNOMRIwEAYDVQQIDAlDaG9uZ1FpbmcxDTALBgNVBAoMBGVkZ2UxDTALBgNV\r\n"
"BAMMBGVkZ2UwHhcNMjAwMzI3MDkxNDIyWhcNNDAwMzIyMDkxNDIyWjA/MQswCQYD\r\n"
"VQQGEwJDTjESMBAGA1UECAwJQ2hvbmdRaW5nMQ0wCwYDVQQKDARlZGdlMQ0wCwYD\r\n"
"VQQDDARlZGdlMIICIjANBgkqhkiG9w0BAQEFAAOCAg8AMIICCgKCAgEAxZOLYLAN\r\n"
"fd7EBR230BAOj8w4abtlNJWWj8bujxSKffwNT0z3oBbYExaDZx6pVm8A6zldQWWi\r\n"
"6On0IqzCPLE8mLH4PdN0smGIsb5HI+qSf/VSdL7ao2JETII98W7p4J25VDtQXvFW\r\n"
"nQ8HxHKeQQUidV4jSJBMFM31/jirCXbxZzX404o+e5nO4BdUmpDtJqJgBUx3O3pH\r\n"
"Fm4q8CX1QpWs34J32rVpzZ33QrhNma3AOLqUBHGs7IuPEST5jBYRZCWDjOtdOS2H\r\n"
"vx3Fn/L1yl8TTTg1RSgJvydzJ1GbTgjfkihZ78nGEs7jtes6CPQTWQupkZN1VBxp\r\n"
"XRDiG69y3QJGnhGcSfZ0RDrTg4orvE2u41fvDTn+8dXEeB5V0C6UnBFjiZCir9S5\r\n"
"zSn421ms5OQJdHoNNKy3Ca0CZ0zrD/dQZcODlBdCA7+NZsJQK6c/MHxIvRZFdm2M\r\n"
"hpaq+zkeNniANQ9cspeIU0TYa+9QL90warsIt6T3DDiy2eYRjM0BH7yXhjCG0Y9z\r\n"
"+EaL4RE08OWkmPq1um2f1cmoOfDAuQ6WG8TyBYNz8I1nW3R8EsmHEhlhbm659Wxx\r\n"
"kB5DyNmcx6+3G1pSJizN2/rjA0D+nWowYiM2osS8A4vEEmC7DqpXYjNtB0Jtx9/q\r\n"
"rrqEL0oQSfNtZ5uNKA4MWhHAMWg0Ey+9n8UCAwEAAaNjMGEwHQYDVR0OBBYEFPq/\r\n"
"GiiUkm9SQ37P8ZTsP055+IrbMB8GA1UdIwQYMBaAFPq/GiiUkm9SQ37P8ZTsP055\r\n"
"+IrbMA8GA1UdEwEB/wQFMAMBAf8wDgYDVR0PAQH/BAQDAgGGMA0GCSqGSIb3DQEB\r\n"
"CwUAA4ICAQAshdi2UuZM1YOsqylZvVCGe2kKdUlKyTITzxNsoW8YgSOzKxchyO/Q\r\n"
"BuSM8ZsjN/gUZei6tKtOn/m2h7i5Ee8seVjKf7Z8YnBD9EkHTOLn3ZrbFnE6sNGJ\r\n"
"rnWh41SsfAGGqRoWhJo6qMSdLujbSjqkK/517MBeejwVBDZae+eJ3tZGASbsRUfI\r\n"
"0yyWJizQWitH+TlvtgrLpI+DYonKsDGLP3bIhzJHXE4WEJDmyCpBsW37kg0xzd61\r\n"
"iWjrhmsfemjMjqJvZuFomvdoPrywh0ujREBOcALwux2FnzyWGCFsuHAw99vMYoR6\r\n"
"LlQf1UFvFc11Qlr/CIMrgDC53khUz31NEwZsBEZ1tMPm+VeJIwiTfj7nhsykwksE\r\n"
"1tfiysndUYIoc+eb9w6sE9lR6nROh13mT9gYerOfgdy7uB+bkuT3orSCfGz4wGrd\r\n"
"qPp/3kPO/MdmCBYMLlAV8MP2FvEOWThYKJTNh7maDi5oaP/Nmpa754uhRvAamroj\r\n"
"jc73xPWYGuIjMepKxwOLPEuVtQN83eVPIof0yiDkkosyEJLvuaigHMI64ihezG5l\r\n"
"aQ0Q4lQDMoYnWD8LU5TszafEuAmt1XvR5PC7c2lnOQehxI6TbmjEnuAOMBC+ASXj\r\n"
"gIkXZ+gHOgVFxrJHP9EUT6crPVcmjWVmDovvX90OCE76dFdo/EHg3w==\r\n"
"-----END CERTIFICATE-----\r\n"
};


void TimerInit(Timer* timer)
{
	timer->end_time = (struct timeval){0, 0};
}

char TimerIsExpired(Timer* timer)
{
	struct timeval now, res;
	gettimeofday(&now, NULL);
	timersub(&timer->end_time, &now, &res);
	return res.tv_sec < 0 || (res.tv_sec == 0 && res.tv_usec <= 0);
}


void TimerCountdownMS(Timer* timer, unsigned int timeout)
{
	struct timeval now;
	gettimeofday(&now, NULL);
	struct timeval interval = {timeout / 1000, (timeout % 1000) * 1000};
	timeradd(&now, &interval, &timer->end_time);
}


void TimerCountdown(Timer* timer, unsigned int timeout)
{
	struct timeval now;
	gettimeofday(&now, NULL);
	struct timeval interval = {timeout, 0};
	timeradd(&now, &interval, &timer->end_time);
}


int TimerLeftMS(Timer* timer)
{
	struct timeval now, res;
	gettimeofday(&now, NULL);
	timersub(&timer->end_time, &now, &res);
	//printf("left %d ms\n", (res.tv_sec < 0) ? 0 : res.tv_sec * 1000 + res.tv_usec / 1000);
	return (res.tv_sec < 0) ? 0 : res.tv_sec * 1000 + res.tv_usec / 1000;
}


int linux_read(Network* n, unsigned char* buffer, int len, int timeout_ms)
{
	struct timeval interval = {timeout_ms / 1000, (timeout_ms % 1000) * 1000};
	if (interval.tv_sec < 0 || (interval.tv_sec == 0 && interval.tv_usec <= 0))
	{
		interval.tv_sec = 0;
		interval.tv_usec = 100;
	}

	setsockopt(n->my_socket, SOL_SOCKET, SO_RCVTIMEO, (char *)&interval, sizeof(struct timeval));

	int bytes = 0;
	while (bytes < len)
	{
		int rc = recv(n->my_socket, &buffer[bytes], (size_t)(len - bytes), 0);
		if (rc == -1)
		{
			if (errno != EAGAIN && errno != EWOULDBLOCK)
			  bytes = -1;
			break;
		}
		else if (rc == 0)
		{
			bytes = 0;
			break;
		}
		else
			bytes += rc;
	}
	return bytes;
}


int linux_write(Network* n, unsigned char* buffer, int len, int timeout_ms)
{
	struct timeval tv;

	tv.tv_sec = 0;  /* 30 Secs Timeout */
	tv.tv_usec = timeout_ms * 1000;  // Not init'ing this can cause strange errors

	setsockopt(n->my_socket, SOL_SOCKET, SO_SNDTIMEO, (char *)&tv,sizeof(struct timeval));
	int	rc = write(n->my_socket, buffer, len);
	return rc;
}

int ca_cert_init(Network* n)
{
	if(n)
	{
		n->server_cert = ca_cert;
		n->server_cert_len = sizeof(ca_cert);
#ifdef USE_X509_AUTH
		n->client_cert = client_cert;
		n->client_cert_len = sizeof(client_cert);
		n->client_pk = client_pk;
		n->client_pk_len = sizeof(client_pk);
#endif
		if(!n->tls)
		{
			n->tls = malloc(sizeof(tls_t));
			if(!n->tls)
			{
				printf("n->tls is NULL");
				return -1;
			}
			memset(n->tls,0,sizeof(tls_t));
			
		}
		return 0;
	}
	return -1;
}

int tls_linux_read(Network* n, unsigned char* buffer, int len, int timeout_ms)
{
	tls_t *tls = (tls_t *)n->tls;
	if(tls)
	{
		int ret = mbedtls_ssl_read(&tls->ssl_ctx, buffer, len);
		return ret;
	}
	return -1;
}
int tls_linux_write(Network* n, unsigned char* buffer, int len, int timeout_ms)
{
	tls_t *tls = (tls_t *)n->tls;
	if(tls)
	{
		int ret = mbedtls_ssl_write(&tls->ssl_ctx, buffer, len);	
		return ret;
	}
	return -1;
}

void NetworkInit(Network* n)
{
#ifdef USE_TLS
	n->mqttread = tls_linux_read;
	n->mqttwrite = tls_linux_write;
#else
	n->my_socket = 0;
	n->mqttread = linux_read;
	n->mqttwrite = linux_write;
#endif
}

static void mytls_debug( void *ctx, int level, const char *file, int line, const char *str )
{  
    DBG("%s\r\n", str);
}
static int tls_tcp_connect(Network* n, char* host, int port) 
{
	int authmode = MBEDTLS_SSL_VERIFY_NONE;
	int ret = 0;
	int flags = 0;
	char porta[10] = {0};
	const char *pers = "mqtt";
	tls_t *ssl;
	
	if(ca_cert_init(n) != 0)
	{
		ret = -1;
		goto exit;
	}
	ssl = (tls_t *)n->tls;
	if(!ssl)
	{
		ret = -1;
		goto exit;
	}
    if (n->server_cert)
       //authmode = MBEDTLS_SSL_VERIFY_REQUIRED;
        authmode = MBEDTLS_SSL_VERIFY_NONE;
	/*
     * Initialize the RNG and the session data
     */
#if defined(MBEDTLS_DEBUG_C)
    mbedtls_debug_set_threshold(0);
#endif
	mbedtls_net_init(&ssl->net_ctx);
    mbedtls_ssl_init(&ssl->ssl_ctx);
    mbedtls_ssl_config_init(&ssl->ssl_conf);
    mbedtls_x509_crt_init(&ssl->cacert);
    mbedtls_x509_crt_init(&ssl->clicert);
    mbedtls_pk_init(&ssl->pkey);
    mbedtls_ctr_drbg_init(&ssl->ctr_drbg);
    mbedtls_entropy_init(&ssl->entropy); 
	if ((ret = mbedtls_ctr_drbg_seed(&ssl->ctr_drbg,
               mbedtls_entropy_func, 
               &ssl->entropy,
               (const unsigned char*)pers,
               strlen(pers))) != 0)
   {
		printf( " failed\n  ! mbedtls_ctr_drbg_seed returned %d\n", ret );
		goto exit;
   }
	/*
	* Load the Client certificate
	*/
	if (n->client_cert && n->client_pk) {
	    ret = mbedtls_x509_crt_parse(&ssl->clicert, (const unsigned char *)n->client_cert, n->client_cert_len);        
	    if (ret < 0) {
	        DBG("Loading cli_cert failed! mbedtls_x509_crt_parse returned -0x%x.", -ret);
	        goto exit;
	    }
	    ret = mbedtls_pk_parse_key(&ssl->pkey, (const unsigned char *)n->client_pk, n->client_pk_len, NULL, 0);                 
	    if (ret != 0) {
	        DBG("failed! mbedtls_pk_parse_key returned -0x%x.", -ret);
	        goto exit;
	    }
	}	
    /*
    * Load the trusted CA
    */    
    /* cert_len passed in is gotten from sizeof not strlen */
    if (n->server_cert && ((ret = mbedtls_x509_crt_parse(&ssl->cacert,
                                        (const unsigned char *)n->server_cert,
                                        n->server_cert_len)) < 0)) {
        DBG("mbedtls_x509_crt_parse() failed, value:-0x%x.", -ret);
        ret = -1;
        goto exit;
    }
	 /*
	 * Start the connection
	 */
	
	snprintf(porta, sizeof(porta), "%d", port) ;
    if ((ret = mbedtls_net_connect(&ssl->net_ctx, host, porta, MBEDTLS_NET_PROTO_TCP)) != 0) {
        DBG("failed! mbedtls_net_connect returned %d, port:%s.", ret, porta);
        goto exit;
    }   
    /*
     * Setup stuff
     */
    if ((ret = mbedtls_ssl_config_defaults(&ssl->ssl_conf,
                                           MBEDTLS_SSL_IS_CLIENT,
                                           MBEDTLS_SSL_TRANSPORT_STREAM,
                                           MBEDTLS_SSL_PRESET_DEFAULT)) != 0) {        
        DBG("mbedtls_ssl_config_defaults() failed, value:-0x%x.", -ret);
        ret = -1;
        goto exit;
    }

    memcpy(&ssl->profile, ssl->ssl_conf.cert_profile, sizeof(mbedtls_x509_crt_profile));    
    ssl->profile.allowed_mds = ssl->profile.allowed_mds | MBEDTLS_X509_ID_FLAG(MBEDTLS_MD_MD5);
    mbedtls_ssl_conf_cert_profile(&ssl->ssl_conf, &ssl->profile);
    
    mbedtls_ssl_conf_max_version(&ssl->ssl_conf, MBEDTLS_SSL_MAJOR_VERSION_3, MBEDTLS_SSL_MINOR_VERSION_3);
    mbedtls_ssl_conf_min_version(&ssl->ssl_conf, MBEDTLS_SSL_MAJOR_VERSION_3, MBEDTLS_SSL_MINOR_VERSION_3);
	mbedtls_ssl_conf_authmode( &ssl->ssl_conf, authmode );
	mbedtls_ssl_conf_ca_chain(&ssl->ssl_conf, &ssl->cacert, NULL);
    mbedtls_ssl_conf_read_timeout(&ssl->ssl_conf,100);
    if ((ret = mbedtls_ssl_conf_own_cert(&ssl->ssl_conf, &(ssl->clicert), &(ssl->pkey))) != 0) {
	    printf(" failed\n  ! mbedtls_ssl_conf_own_cert returned %d\n", ret);
	    return ret;
    }
	mbedtls_ssl_conf_rng(&ssl->ssl_conf, mbedtls_ctr_drbg_random, &ssl->ctr_drbg);
	mbedtls_ssl_conf_dbg(&ssl->ssl_conf, mytls_debug, NULL);
    if ((ret= mbedtls_ssl_setup(&ssl->ssl_ctx, &ssl->ssl_conf)) != 0) {
        DBG("mbedtls_ssl_setup() failed, value:-0x%x.", -ret);
        ret = -1;
        goto exit;
    }  
    //mbedtls_ssl_set_hostname
    //if( ( ret = mbedtls_ssl_set_hostname( &ssl->ssl_ctx, "10.12.6.24" ) ) != 0 )   
    //{        
	//	DBG( " failed\n  ! mbedtls_ssl_set_hostname returned %d\n\n", ret );        
	//	goto exit;    
    //}
	mbedtls_ssl_set_bio(&ssl->ssl_ctx, &ssl->net_ctx, mbedtls_net_send, mbedtls_net_recv, mbedtls_net_recv_timeout);
    /*
    * Handshake
    */
    while ((ret = mbedtls_ssl_handshake(&ssl->ssl_ctx)) != 0) {
        if (ret != MBEDTLS_ERR_SSL_WANT_READ && ret != MBEDTLS_ERR_SSL_WANT_WRITE) {      
            DBG("mbedtls_ssl_handshake() failed, ret:-0x%x.\r\n", -ret);
            ret = -1;
            goto exit;
        }
    }
    /*
     * Verify the server certificate
     */
    /* In real life, we would have used MBEDTLS_SSL_VERIFY_REQUIRED so that the
        * handshake would not succeed if the peer's cert is bad.  Even if we used
        * MBEDTLS_SSL_VERIFY_OPTIONAL, we would bail out here if ret != 0 */
    if ((flags = mbedtls_ssl_get_verify_result(&ssl->ssl_ctx)) != 0) {
        char vrfy_buf[512];
        DBG("svr_cert varification failed.");
        mbedtls_x509_crt_verify_info(vrfy_buf, sizeof(vrfy_buf), "  ! ", flags);
        DBG("%s", vrfy_buf);
    }
    else
        DBG("svr_cert varification ok.\r\n"); 
exit:	
    return ret;
}

int NetworkConnect(Network* n, char* addr, int port)
{
#ifdef USE_TLS
	return tls_tcp_connect(n, addr, port);
#else
	int type = SOCK_STREAM;
	struct sockaddr_in address;
	int rc = -1;
	sa_family_t family = AF_INET;
	struct addrinfo *result = NULL;
	struct addrinfo hints = {0, AF_UNSPEC, SOCK_STREAM, IPPROTO_TCP, 0, NULL, NULL, NULL};

	if ((rc = getaddrinfo(addr, NULL, &hints, &result)) == 0)
	{
		struct addrinfo* res = result;

		/* prefer ip4 addresses */
		while (res)
		{
			if (res->ai_family == AF_INET)
			{
				result = res;
				break;
			}
			res = res->ai_next;
		}

		if (result->ai_family == AF_INET)
		{
			address.sin_port = htons(port);
			address.sin_family = family = AF_INET;
			address.sin_addr = ((struct sockaddr_in*)(result->ai_addr))->sin_addr;
		}
		else
			rc = -1;

		freeaddrinfo(result);
	}

	if (rc == 0)
	{
		n->my_socket = socket(family, type, 0);
		if (n->my_socket != -1)
			rc = connect(n->my_socket, (struct sockaddr*)&address, sizeof(address));
		else
			rc = -1;
	}

	return rc;
#endif
}

void NetworkDisconnect(Network* n)
{
#ifdef USE_TLS
	tls_t *ssl = n->tls;
    if (!ssl)
        return;
    
    mbedtls_ssl_close_notify(&ssl->ssl_ctx);
    mbedtls_net_free(&ssl->net_ctx);
    mbedtls_x509_crt_free(&ssl->cacert);
    mbedtls_x509_crt_free(&ssl->clicert);
    mbedtls_pk_free(&ssl->pkey);
    mbedtls_ssl_free(&ssl->ssl_ctx);    
    mbedtls_ssl_config_free(&ssl->ssl_conf);
    mbedtls_ctr_drbg_free(&ssl->ctr_drbg);
    mbedtls_entropy_free(&ssl->entropy);
    
    free(ssl);	
#else
	close(n->my_socket);
#endif
}
