#include <pthread.h>
#include <stdlib.h>
#include <string.h>

#include "../OneNETMqttClient/OneNETMqttClient.h"
#include "../PahoMQTTSDK/Driver/linux/MQTTLinux.h"
#include "../OneNETMqttClient/cJSON/cJSON.h"

typedef struct TCP_Network
{
	int my_socket;
	int (*tcpread) (struct TCP_Network*, unsigned char*, int, int);
	int (*tcpwrite) (struct TCP_Network*, unsigned char*, int, int);
	const char *server_cert;        /**< Server certification. */
    const char *client_cert;        /**< Client certification. */
    const char *client_pk;          /**< Client private key. */
    int server_cert_len;            /**< Server certification lenght, server_cert buffer size. */
    int client_cert_len;            /**< Client certification lenght, client_cert buffer size. */
    int client_pk_len;              /**< Client private key lenght, client_pk buffer size. */
	void *tls;
} TCP_Network;

void init_network(TCP_Network *n)
{
	n.my_socket = -1;
#ifdef USE_TLS
	n->tcpread = tls_linux_read;
	n->tcpwrite = tls_linux_write;
#else
	n->tcpread = linux_read;
	n->tcpwrite = linux_write;
#endif
}

#define TCP_DEVICE_ID "10237865"
#define TCP_PRODUCT_ID "100801"
#define TCP_DEVICESECRET "Y2Q4ZjlhYzM5OTRlNDJhZjI3NGQ="
#define TCP_SERVER_IP "10.12.6.24"
#define TCP_SERVER_PORT 30372

char str_val[100] = "abc";
int int_val = 32;
long long_val = 43;
bool bool_val = false;
double double_val = 32.5;
unsigned char *base64_encode(unsigned char *str,int length)
{
	long len;
	long str_len;
	unsigned char *res;
	int i, j;
	//定义base64编码表  
	unsigned char *base64_table = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";

	//计算经过base64编码后的字符串长度  
	//str_len = strlen(str);
	str_len = length;
	if (str_len % 3 == 0)
		len = str_len / 3 * 4;
	else
		len = (str_len / 3 + 1) * 4;

	res = malloc(sizeof(unsigned char)*len + 1);
	res[len] = '\0';

	//以3个8位字符为一组进行编码  
	for (i = 0, j = 0; i < len - 2; j += 3, i += 4)
	{
		res[i] = base64_table[str[j] >> 2]; //取出第一个字符的前6位并找出对应的结果字符  
		res[i + 1] = base64_table[(str[j] & 0x3) << 4 | (str[j + 1] >> 4)]; //将第一个字符的后位与第二个字符的前4位进行组合并找到对应的结果字符  
		res[i + 2] = base64_table[(str[j + 1] & 0xf) << 2 | (str[j + 2] >> 6)]; //将第二个字符的后4位与第三个字符的前2位组合并找出对应的结果字符  
		res[i + 3] = base64_table[str[j + 2] & 0x3f]; //取出第三个字符的后6位并找出结果字符  
	}

	switch (str_len % 3)
	{
	case 1:
		res[i - 2] = '=';
		res[i - 1] = '=';
		break;
	case 2:
		res[i - 1] = '=';
		break;
	}

	return res;
}
static void _hex2str(unsigned char *input, unsigned short input_len, char *output)
{
	char *zEncode = "0123456789ABCDEF";
	int i = 0, j = 0;

	for (i = 0; i < input_len; i++) {
		output[j++] = zEncode[(input[i] >> 4) & 0xf];
		output[j++] = zEncode[(input[i]) & 0xf];
	}
}
unsigned char *base64_decode(unsigned char *code)
{
	//根据base64表，以字符找到对应的十进制数据  
	int table[] = { 0,0,0,0,0,0,0,0,0,0,0,0,
			 0,0,0,0,0,0,0,0,0,0,0,0,
			 0,0,0,0,0,0,0,0,0,0,0,0,
			 0,0,0,0,0,0,0,62,0,0,0,
			 63,52,53,54,55,56,57,58,
			 59,60,61,0,0,0,0,0,0,0,0,
			 1,2,3,4,5,6,7,8,9,10,11,12,
			 13,14,15,16,17,18,19,20,21,
			 22,23,24,25,0,0,0,0,0,0,26,
			 27,28,29,30,31,32,33,34,35,
			 36,37,38,39,40,41,42,43,44,
			 45,46,47,48,49,50,51
	};
	long len;
	long str_len;
	unsigned char *res;
	int i, j;

	//计算解码后的字符串长度  
	len = strlen(code);
	//判断编码后的字符串后是否有=  
	if (strstr(code, "=="))
		str_len = len / 4 * 3 - 2;
	else if (strstr(code, "="))
		str_len = len / 4 * 3 - 1;
	else
		str_len = len / 4 * 3;

	res = malloc(sizeof(unsigned char)*str_len + 1);
	res[str_len] = '\0';

	//以4个字符为一位进行解码  
	for (i = 0, j = 0; i < len - 2; j += 3, i += 4)
	{
		res[j] = ((unsigned char)table[code[i]]) << 2 | (((unsigned char)table[code[i + 1]]) >> 4); //取出第一个字符对应base64表的十进制数的前6位与第二个字符对应base64表的十进制数的后2位进行组合  
		res[j + 1] = (((unsigned char)table[code[i + 1]]) << 4) | (((unsigned char)table[code[i + 2]]) >> 2); //取出第二个字符对应base64表的十进制数的后4位与第三个字符对应bas464表的十进制数的后4位进行组合  
		res[j + 2] = (((unsigned char)table[code[i + 2]]) << 6) | ((unsigned char)table[code[i + 3]]); //取出第三个字符对应base64表的十进制数的后2位与第4个字符进行组合  
	}

	return res;

}

int CalcAuthInfo(char *data, char* did, char* pid, char* device_secret)
{
	char token[100] = { 0 };
	char didpid[100] = { 0 };
	strcat(didpid, did);
	strcat(didpid, "&");
	strcat(didpid, pid);
	extern void utils_hmac_sha1(const char* msg, int msg_len, char* digest, const char* key, int key_len);
	utils_hmac_sha1(didpid, strlen(didpid), token, device_secret, strlen(device_secret));
	unsigned char* buf = NULL;
	buf = base64_encode(token, 20);
	sprintf(data, "*%s#%s#%s*", pid, did, buf);
	printf("Auth info is:%s\r\n", data);
	free(buf);

	return 0;
}

int commandHandler(char* result, char* payload) {
	char* uuid;
	char* function_type;

	cJSON* req = NULL;
	req = cJSON_Parse(payload);

	cJSON* resp = cJSON_CreateObject();
	char* resp_str;

	cJSON* item;
	item = cJSON_GetObjectItem(req, "uuid");
	if (item != NULL) {
		uuid = item->valuestring;
		cJSON_AddStringToObject(resp, "uuid", uuid);
	}
	else {
		printf("cmdreq error : no uuid!\n");
		return;
	}
	item = cJSON_GetObjectItem(req, "functionType");
	if (item != NULL) {
		function_type = item->valuestring;
	}
	else {
		printf("cmdreq error : no functionType!\n");
		return;
	}
	item = cJSON_GetObjectItem(req, "params");
	cJSON* param;
	cJSON* identifierValue = cJSON_CreateObject();
	if (item != NULL) {
		if (strcmp(function_type, "propertyGet") == 0) {
			int array_size = cJSON_GetArraySize(item);
			int i = 0;
			for (i = 0; i < array_size; i++) {
				char* value_name;
				param = cJSON_GetArrayItem(item, i);
				if (param != NULL)
				{
					value_name = param->valuestring;
				}
				else {
					continue;
				}
				if (strcmp(value_name, "str_val") == 0) {
					cJSON_AddStringToObject(identifierValue, "str_val", str_val);
				}
				else if (strcmp(value_name, "int_val") == 0) {
					cJSON_AddNumberToObject(identifierValue, "int_val", int_val);
				}
				else if (strcmp(value_name, "double_val") == 0) {
					cJSON_AddNumberToObject(identifierValue, "double_val", double_val);
				}
				else if (strcmp(value_name, "long_val") == 0) {
					cJSON_AddNumberToObject(identifierValue, "long_val", long_val);
				}
				else if (strcmp(value_name, "bool_val") == 0) {
					cJSON_AddBoolToObject(identifierValue, "bool_val", bool_val);
				}
			}
			cJSON_AddItemToObject(resp, "identifierValue", identifierValue);

		}
		else if (strcmp(function_type, "propertySet") == 0) {
			cJSON* value;
			char* value_name;
			value = cJSON_GetObjectItem(item, "int_val");
			if (value != NULL) {
				int_val = value->valueint;
			}
			value = cJSON_GetObjectItem(item, "double_val");
			if (value != NULL) {
				double_val = value->valuedouble;
			}
			value = cJSON_GetObjectItem(item, "long_val");
			if (value != NULL) {
				long_val = value->valueint;
			}
			value = cJSON_GetObjectItem(item, "bool_val");
			if (value != NULL) {
				bool_val = value->valueint;
			}
		}
		else {
			printf("cmdreq error : error function type %s!\n", function_type);
			return;
		}
	}
	else {
		printf("cmdreq error : no params!\n");
		return;
	}
	resp_str = cJSON_PrintUnformatted(resp);

	int a = strlen(resp_str);
	unsigned char len[2] = {0};
	len[0] = (unsigned char)(a >> 8);
	len[1] = (unsigned char)(a);

	result[0] = '#';
	result[1] = len[0];
	result[2] = len[1];
	memcpy(result + 3, resp_str, strlen(resp_str));

	printf("command resp; %c %d %d %s \n", result[0], result[1], result[2], result+3);
	
	return a + 3;
}

int uploadDeviceData(char* device_data) {
	cJSON* item = cJSON_CreateObject();

	cJSON_AddNumberToObject(item, "double_val", double_val);
	cJSON_AddStringToObject(item, "str_val", str_val);
	cJSON_AddNumberToObject(item, "int_val", int_val);
	cJSON_AddNumberToObject(item, "long_val", long_val);
	cJSON_AddBoolToObject(item, "bool_val", bool_val);
	
	char* device_data_str;
	device_data_str = cJSON_PrintUnformatted(item);

	int a = strlen(device_data_str);
	unsigned char len[2] = { 0 };
	len[0] = (unsigned char)(a >> 8);
	len[1] = (unsigned char)(a);

	device_data[0] = len[0];
	device_data[1] = len[1];
	memcpy(device_data + 2, device_data_str, strlen(device_data_str));

	printf("device upload data; %d %d %s \n", device_data[0], device_data[1], device_data + 2);

	return a + 2;

}

pthread_t tcp_RecvData_id; 
TCP_Network n;
void *tcp_recv_handler(void* __this)
{
    char readbuf[1000] = {0};
    int bytes = 0;
    while(1)
    {
        sleep(1);
        memset(readbuf,0,sizeof(readbuf));
        bytes = n.tcpread(&n,readbuf,sizeof(readbuf),500);
        if(bytes)
        {
            printf("Recv %d bytes data:%s\r\n",bytes,readbuf);
			char result[300] = {0};
			int a = commandHandler(result,readbuf);
			n.tcpwrite(n.my_socket, result, a);
        }
        else
        {
            printf("Recv data nothing\r\n");
        }
        
    }
}

void main(int argc, char *argv[])
{
	char* server_ip;
	int server_port;
	char* did;
	char* pid;
	char* device_secret;
	if (argc > 1) 
	{
		server_ip = argv[1];
		server_port = atoi(argv[2]);
		did = argv[3];
		pid = argv[4];
		device_secret = argv[5];
	}
	else
	{
		server_ip = TCP_SERVER_IP;
		server_port = TCP_SERVER_PORT;
		did = TCP_DEVICE_ID;
		pid = TCP_PRODUCT_ID;
		device_secret = TCP_DEVICESECRET;
	}
	
    char readbuf[100] = {0};
    char AuthInfo[50] = {0};

	printf("port %d \n", server_port);
	init_network();
    if(NetworkConnect(&n, server_ip, server_port))
    {
        printf("NetworkConnect error");
    }

    CalcAuthInfo(AuthInfo,did,pid,device_secret);
    n.tcpwrite(n.my_socket,AuthInfo,strlen(AuthInfo));
//waitresponse:
    if(n.tcpread(n.my_socket,readbuf,sizeof(readbuf)))
    {
        if(readbuf[0] == 0x06 && readbuf[1] == 0x06)
        {
            printf("Connect2Server Succ\r\n");
        }
        else
        {
            printf("Connect2Server error\r\n");
            return;
        }
    }
	else
	{
		//goto waitresponse;
	}
	
     
    pthread_create(&tcp_RecvData_id, NULL, tcp_recv_handler, NULL);
    while(1)
    {
        char keep_alive_data[4] = {94,94,94,94};
        sleep(10);
        if(n.tcpwrite(n.my_socket,keep_alive_data,strlen(keep_alive_data)))
        {
            //printf("send keep_alive_data...\r\n");
        }

		int len;
		char device_data[300] = { 0 };
		len = uploadDeviceData(device_data);
		n.tcpwrite(n.my_socket, device_data, len);
    }
}